Package: futile.logger
Type: Package
Title: A Logging Utility for R
Version: 1.4.9
Date: 2025-12-22
Maintainer: Brian Lee Yung Rowe <r@zatonovo.com>
Authors@R: 
  person(given=c("Brian", "Lee", "Yung"), family="Rowe",
    role=c("aut", "cre"), email="r@zatonovo.com")
Depends: R (>= 3.0.0)
Imports: utils, lambda.r (>= 1.1.0), futile.options
Suggests: testit, jsonlite, httr, crayon, rsyslog, glue
Description: Provides a simple yet powerful logging utility. Based loosely on
    log4j, futile.logger takes advantage of R idioms to make logging a
    convenient and easy to use replacement for cat and print statements.
License: LGPL-3
LazyLoad: yes
NeedsCompilation: no
ByteCompile: yes
Collate: 'options.R' 'appender.R' 'constants.R' 'layout.R' 'logger.R'
        'scat.R' 'util.R' 'futile.logger-package.R'
RoxygenNote: 7.1.2
URL: https://github.com/zatonovo/futile.logger
Packaged: 2025-12-22 17:34:15 UTC; brian
Author: Brian Lee Yung Rowe [aut, cre]
Repository: CRAN
Date/Publication: 2025-12-29 08:50:01 UTC
Built: R 4.6.0; ; 2026-04-25 23:53:51 UTC; unix
