#' Constants for 'futile.logger'
#' 
#' Log level constants and the logger options.
#'
#' The logging configuration is managed by 'logger.options', a function
#' generated by OptionsManager within 'futile.options'.
#' 
#' @name logger.options
#' @aliases FATAL ERROR WARN INFO DEBUG TRACE
#' @usage logger.options(..., simplify = FALSE, update = list())
#' @param ... TODO
#' @param simplify TODO
#' @param update TODO
#' @author Brian Lee Yung Rowe
#' @seealso \code{futile.options}
#' @keywords data
logger.options <- OptionsManager('logger.options')

